'use strict';

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _mapKey, _compareKeys, _emptySet, _map;
const defaultOptions = {
  mapKey: (a) => a,
  compareKeys: null
  // biome-ignore lint/suspicious/noExplicitAny: any is needed for type inference
};
class CompositeKeyMultiMap {
  constructor(options = {}) {
    // biome-ignore lint/suspicious/noExplicitAny: any is needed for type inference
    __privateAdd(this, _mapKey, void 0);
    __privateAdd(this, _compareKeys, void 0);
    __privateAdd(this, _emptySet, /* @__PURE__ */ new Set());
    __privateAdd(this, _map, /* @__PURE__ */ new Map());
    __privateSet(this, _mapKey, options.mapKey ?? defaultOptions.mapKey);
    __privateSet(this, _compareKeys, options.compareKeys ?? defaultOptions.compareKeys);
  }
  set(keys, value) {
    let map = __privateGet(this, _map);
    const chain = __privateGet(this, _compareKeys) ? keys.toSorted(__privateGet(this, _compareKeys)) : keys;
    for (let i = 0; i < chain.length; i++) {
      const key = __privateGet(this, _mapKey).call(this, chain[i]);
      let next = map.get(key);
      if (!next) {
        next = [/* @__PURE__ */ new Set(), /* @__PURE__ */ new Map()];
        map.set(key, next);
      }
      if (i === chain.length - 1) {
        next[0].add(value);
      } else {
        map = next[1];
      }
    }
  }
  get(keys) {
    let map = __privateGet(this, _map);
    const chain = __privateGet(this, _compareKeys) ? keys.toSorted(__privateGet(this, _compareKeys)) : keys;
    for (let i = 0; i < chain.length; i++) {
      const key = __privateGet(this, _mapKey).call(this, chain[i]);
      const next = map.get(key);
      if (!next) {
        return __privateGet(this, _emptySet);
      }
      if (i === chain.length - 1) {
        return next[0];
      }
      map = next[1];
    }
    return __privateGet(this, _emptySet);
  }
  delete(keys) {
    let map = __privateGet(this, _map);
    const chain = __privateGet(this, _compareKeys) ? keys.toSorted(__privateGet(this, _compareKeys)) : keys;
    for (let i = 0; i < chain.length; i++) {
      const key = __privateGet(this, _mapKey).call(this, chain[i]);
      const next = map.get(key);
      if (!next) {
        return false;
      }
      if (i === chain.length - 1) {
        if (next[0].size > 0) {
          next[0].clear();
          return true;
        }
      }
      map = next[1];
    }
    return false;
  }
  has(keys) {
    let map = __privateGet(this, _map);
    const chain = __privateGet(this, _compareKeys) ? keys.toSorted(__privateGet(this, _compareKeys)) : keys;
    for (let i = 0; i < chain.length; i++) {
      const key = __privateGet(this, _mapKey).call(this, chain[i]);
      const next = map.get(key);
      if (!next) {
        return false;
      }
      if (i === chain.length - 1) {
        return next[0].size > 0;
      }
      map = next[1];
    }
    return false;
  }
  clear() {
    __privateGet(this, _map).clear();
  }
  *keys() {
    const stack = [
      [__privateGet(this, _map), []]
    ];
    while (stack.length > 0) {
      const [map, keys] = stack.pop();
      for (const [key, [value, next]] of map) {
        if (value !== void 0) {
          yield keys.concat(key);
        }
        stack.push([next, keys.concat(key)]);
      }
    }
  }
  *values() {
    const stack = [__privateGet(this, _map)];
    while (stack.length > 0) {
      const map = stack.pop();
      for (const [, [value, next]] of map) {
        if (value !== void 0) {
          yield value;
        }
        stack.push(next);
      }
    }
  }
  *entries() {
    const stack = [
      [__privateGet(this, _map), []]
    ];
    while (stack.length > 0) {
      const [map, keys] = stack.pop();
      for (const [key, [value, next]] of map) {
        if (value !== void 0) {
          yield [keys.concat(key), value];
        }
        stack.push([next, keys.concat(key)]);
      }
    }
  }
}
_mapKey = new WeakMap();
_compareKeys = new WeakMap();
_emptySet = new WeakMap();
_map = new WeakMap();

exports.CompositeKeyMultiMap = CompositeKeyMultiMap;
